/************************************************************
 *  GAI - Temp                                              *
 *                                                          *
 *  A temperature displaying applet using GAI               *
 *                                                          *
 ************************************************************/

/************************************************************
 *  Released unter GPL                                      *
 ************************************************************/


#include "gai-temp.h"



/************************************************************
 * static types                                             *
 ************************************************************/

static char *orientation_names[] = {_("horizontal"), _("vertical"), NULL};
static char *temperature_unit_names[] = { _("celsius"), _("fahrenheit"), _("kelvin"), NULL};

static gt_setup setup;

static GaiPI *gai_temp_pref;

static GaiSSF spinfloat_temp_limits = {20.0, 100.0, 0.5, 1};

static GaiSS  spin_temps_at_once_limits = {1, 255, 1};


static GaiPI base_pref[] = {{GAI_NOTEBOOK, _("Common")},
			    {GAI_FRAME, _("Color")},
			    {GAI_COLORSELECTOR, _("normal temperature"),
			     &setup.col_okay, &setup.col_okay},
			    {GAI_COLORSELECTOR,_("high temperature"), 
			     &setup.col_high, &setup.col_high},
			    {GAI_COLORSELECTOR, _("critical temperature"), 
			     &setup.col_critical, &setup.col_critical},
			    {GAI_COLORSELECTOR, _("background color"), 
			     &setup.col_bg, &setup.col_bg},
			    {GAI_CHECKBUTTON, 
			     _("use gtk background\n(the only possibility in gnome-panel)"), 
			     &setup.col_bg_use_gtk, &setup.col_bg_use_gtk},
			    {GAI_FRAME_E},
   			    {GAI_FRAME_R, _("Temperature")},
		         {GAI_RADIOBUTTON, temperature_unit_names, &setup.temp_unit, &setup.temp_unit},
    			    {GAI_FRAME_E},			    
			    {GAI_FRAME, _("Layout")},
			    {GAI_BUTTON_STOCK | GAI_LEFT, _("choose font"), 
			     GTK_STOCK_SELECT_FONT, (void *)select_font},

			    {GAI_CHECKBUTTON, 
			     _("use the default gtk font"), 
			     &setup.use_default_font,&setup.use_default_font}, 


			    {GAI_SPINBUTTON, _("number of temperatures"), 
			     &setup.temps_at_once, &setup.temps_at_once, 
			     &spin_temps_at_once_limits},

			    {GAI_TEXT, _("orientation")},

			    {GAI_RADIOBUTTON, orientation_names, 
			     &setup.orientation, &setup.orientation},

			    {GAI_FRAME_E},
			    {GAI_NOTEBOOK_E},
			    {GAI_END}};



static GdkPixbuf *img_default [TEMP_SOURCES+1];

static char *font_selected; /* font selected in the font dialog */

static FILE *stdlog;  /* file descriptor which is finally used in fprintf */



/* store reading functions in an array */

READ_TEMP_FUNC_T  read_temp_source [TEMP_SOURCES] =
                         { &read_cpu_temp_acpi, 
			   &read_hdd_temp,
			   &read_lm_sensor_temp}; 

/* a tiny debug callback function */
void notify (gpointer data, GObject *obj)
{
    fprintf (stdlog, "GOBJ \"%s\" finalized\n", (char*)data);
}

/************************************************************
 * Function implementation                                  *
 ************************************************************/



int main(int argc, char **argv)
{
    int xy, env;

    /* most important: init gai ;-) */
    env = gai_init2(&applet_defines, &argc, &argv);

    /* choose where to log messages: */

    if (1 == env )
    {
	/* we use GNOME Panel - so send the output to a file */

	stdlog =  fopen ("/tmp/gai-temp.log", "w");
	if (NULL == stdlog)
	{
	    fprintf (stderr, "Can't open /tmp/gai-temp.log for writing.\n");	    
	    stdlog = stderr;
	}
    } /* if gnome-panel-applet */
    else 
    {
	stdlog = stderr; /* write to stderr */
    }
    

    xy = 23; /* gai_get_size () -1 ; 

	     for some reason I get 64 here on gnome, so set it to 23 */

    fprintf (stdlog, "gai_get_size () = %i\n", gai_get_size() );

    /* load the default images */

    img_default[HDDTEMP] = gai_load_image_at_size ("festplatte.svg", xy, xy);

    img_default[ACPI_CPU] = gai_load_image_at_size ("prozessor.svg", xy, xy);
    
    img_default[LM_SENSORS] = gai_load_image_at_size ("sensor.svg", xy, xy);
    
    img_default[NO_SENSORS] = NULL; /* don't load by default */


    font_selected = NULL;

    /* create the preference window structure
       and load the configuration */

    create_pref();
    
    /* where does the applet run ?  write it to the setup-struct */
    
    if (1 == env )
	setup.env = ENV_GNOME_PANEL;
    else
	setup.env = ENV_ELSE;
    

    /* create preference window */

    gai_preferences2(_("GAI Temperature Applet - Preferences"),
		     gai_temp_pref,
		     _("Select  the temperatures to be shown or not,"),
		     (GaiCallback0 *)gai_temp_settings,
		     NULL);

    /* Update two times a second */
    gai_signal_on_update((GaiCallback0 *)update_temp, 2000, NULL);

    /* and start once to create the window  */
    update_temp();

    gai_start();

    g_object_unref (setup.bg);

    return 0;
} /* main */

/************************************************************/


gboolean update_temp ()
{

/* quick overview what it does:

1. create temp_str_pixmap for any temp that is displayed
2. calculate the size needet for all displayed temps
3. create background pixmap (if needet)
4. draw for each temp an icon and temp_str onto the background pixmap
5. unref temp_str_pixmaps

*/
    char *tmp;
    GdkPixbuf *ico;
    GaiColor color;
    int state = 0; /* return val for readtemp_fu */

    int bg_w, bg_h; /* width and height of the new background */

    int i;
    GSList *start = setup.sl_display_pos;


    /* 1. create temp_str_pixmap for any temp that is displayed */

    for (i=0; i < setup.temps_at_once; i++)
    {
	if ( start )
	{
	    tmp = create_temp_str(S_SENSOR_TEMP(start->data));
	    color = get_color_from_temp (S_SENSOR_TEMP(start->data));




	    /* test what font to use */


	    if (setup.use_default_font)
	      {
		setup.temp_img[i].text = gai_text_create_simple 
				        (tmp, color.r, color.g, color.b);
	      }
	    else
	      {
		setup.temp_img[i].text = gai_text_create (tmp,
							  setup.font_name,
							  (setup.font_size>0)?
							  setup.font_size:9,
							  setup.font_feature,
							  color.r,
							  color.g, color.b);
	      }
	    
	    setup.temp_img[i].txt_y = gdk_pixbuf_get_height 
		                        (setup.temp_img[i].text );
	    setup.temp_img[i].txt_x = gdk_pixbuf_get_width 
		                        (setup.temp_img[i].text );

#ifdef DEBUG
	    g_object_weak_ref  (G_OBJECT( setup.temp_img[i].text), 
				notify, "text");
#endif
	    g_free (tmp);

	    ico = setup.temp_img[i].icon = get_icon_from_stemp 
		                        (S_SENSOR_TEMP(start->data));

	    setup.temp_img[i].ico_y = gdk_pixbuf_get_height (ico);
	    setup.temp_img[i].ico_x = gdk_pixbuf_get_width  (ico);


	} /* if start */
	else {
	    /* set every entry to 0 */
	    memset (&setup.temp_img[i], 0, sizeof (temp_image_t));
	} /* else */

	start = g_slist_next (start);
    } /* for */


    /* 2. calculate the size needet for all displayed temps */

    calc_applet_size (&bg_w, &bg_h);


    /* 3. create background pixmap (if necessary) */
    
    if ( !setup.bg || (bg_w > setup.bg_dim.x) || (bg_h > setup.bg_dim.y) )
    {
	fprintf (stdlog, "New applet size: %ix%i\n", bg_w, bg_h);

	/* free the old pixmap*/
	if (setup.bg) { g_object_unref (setup.bg); }

	/* allocate space for the new background pixmap */
	setup.bg = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
                                 TRUE, 8, bg_w, bg_h);

	setup.bg_dim.x = bg_w;
	setup.bg_dim.y = bg_h;

#ifdef DEBUG
	g_object_weak_ref  (G_OBJECT(setup.bg), notify, "background");
#endif
    }

    /* 4. draw for each temp an icon and temp_str onto the background pixmap */

    draw_applet (setup.bg);
    
    gai_background_from_gdkpixbuf (setup.bg,GAI_BACKGROUND_MAX_SIZE_IMAGE);
 
    /* 5. unref temp_str_pixmaps */
    for (i=0; i < setup.temps_at_once; i++)
    {
	if (setup.temp_img[i].text)
	    g_object_unref (setup.temp_img[i].text);
	memset (&setup.temp_img[i], 0, sizeof(temp_image_t));
    } /* for */



    /* move in display list */
    setup.sl_display_pos = g_slist_nth (setup.sl_display_pos, 
					setup.temps_at_once);

    if ( !setup.sl_display_pos)
    {
	setup.sl_display_pos = setup.sl_display; /* reset */

	/* read new sensor values */

	for (i=0; i< TEMP_SOURCES; i++)
	{
	    if (setup.enable_temp_source[i]) 
	    {
		setup.sl_source[i] = read_temp_source[i] (setup.sl_source[i],
							  &state);
		if (WELL_DONE != state) 
		{
		    /* error while reading; disable temp_source */
		    setup.enable_temp_source[i] = 0;		    
		} /* ! WELL_DONE */
	    } /* if */

	} /* for */

    } /* !setup.sl_display_pos */

    return 1;
} /* update_temp */

/************************************************************/

void create_pref ()
{
    int result, i;
    const int min_pref_size =  sizeof(base_pref)/sizeof(GaiPI); 
			/* minimum size of preference_struct */
    int pref_size = min_pref_size;
    const int need_per_nbp   = 5; /* entries per notebook */
    const int need_per_stemp = 2; /* entries per new temp-sensor */
    GaiPI *p = NULL; 
    gchar **vect;


    memset (&setup, 0, sizeof(gt_setup));

    /* find sensors */


    for (i=0; i< TEMP_SOURCES; i++)
    {
	setup.sl_source[i] = read_temp_source[i] (setup.sl_source[i],
						      &result);

	setup.enable_temp_source[i] = (WELL_DONE == result);

    } /* for */
    
    /* load config ;-) */
    load_config ();


    for (i=0; i < TEMP_SOURCES; i++) 
    {
	if (setup.sl_source[i])
	{
	    /* add a preference-page */
	    pref_size += need_per_nbp; /* add place for a notebook-pace */
	    pref_size += g_slist_length (setup.sl_source[i])*need_per_stemp;
	} /* if sl_source[i] */
    } /* for */
    
    p = gai_temp_pref = (GaiPI*) malloc (sizeof (GaiPI)* pref_size);

    memset (gai_temp_pref, 0, sizeof (GaiPI)*pref_size);

    memcpy (p, base_pref, sizeof(GaiPI) * min_pref_size);

    p += min_pref_size-1; /* set it to the end of the notebook */

    for (i=0; i < TEMP_SOURCES; i++) 
    {
	if (setup.sl_source[i])
	{
	    GSList *pst = setup.sl_source[i];

	    /* add a notebook */
	    p->type = GAI_NOTEBOOK;
	    p->name = _(NOTEBOOK_NAMES[i]);
	    p++;

	    /* create the content of the notebook */
	    
	    /* high temp. spinfloat */
	    p->type = GAI_SPINBUTTONFLOAT;
	    p->name = _(TEMPERATURE_NAMES[I_HIGH_TEMP]);
	    p->default_val = &setup.temp_delim[i][I_HIGH_TEMP];
	    p->result_val  = &setup.temp_delim[i][I_HIGH_TEMP];
	    p->extra = &spinfloat_temp_limits;
	    p++;

	    /* critical temp. spinfloat */
	    p->type = GAI_SPINBUTTONFLOAT;
	    p->name = _(TEMPERATURE_NAMES[I_CRITICAL_TEMP]);
	    p->default_val = &setup.temp_delim[i][I_CRITICAL_TEMP];
	    p->result_val  = &setup.temp_delim[i][I_CRITICAL_TEMP];
	    p->extra = &spinfloat_temp_limits;
	    p++;

	    p->type = GAI_HLINE;
	    p++;
	    
	    while (pst)
	    {
		vect = g_strsplit(S_SENSOR_TEMP(pst->data)->dev, DEV_DELIM, 2);
		if (!vect)
		{
		    g_warning ("can't extract the name of the device");
		    p->name = S_SENSOR_TEMP(pst->data)->dev;
		}
		else {
		    p->name = g_strdup (vect[1]);
		    g_strfreev (vect);
		}

		p->type = GAI_CHECKBUTTON;

		p->default_val = &S_SENSOR_TEMP(pst->data)->display;
		p->result_val  = &S_SENSOR_TEMP(pst->data)->display;
		p++;
		
		p->type = GAI_TEXTENTRY;
		p->name = _("alias");
		p->default_val = &S_SENSOR_TEMP(pst->data)->alias;
		p->result_val = &S_SENSOR_TEMP(pst->data)->alias;
		p++;
		
		pst = g_slist_next(pst);

	    } /* for */
	    
	    /* finish the notebook */
	    p->type = GAI_NOTEBOOK_E;
	    p++;

	} /* if sl_source[i] */
    } /* for */
    
    p->type = GAI_END;
} /* create_pref */

/************************************************************/

void load_config ()
{
    const guint32 d_col_okay =  COL_G(150) + COL_B(10) + COL_A(255);
    const guint32 d_col_high =  COL_R(240) + COL_G(180) + COL_A(255);
    const guint32 d_col_crit =  COL_R(201) + COL_A(255);
    const guint32 d_col_bg   = COL_B(224) + COL_R(224) + COL_G(224);

    GList *sensor_list, *p;
    GSList *q;
    char *temp = NULL, *config_key;
    int i, n, k, flags = 0;
    char *flag_start = NULL, *flag_end = NULL;

    const int KEYWORDS = 3;
    const char *keywords[] = {"Regular ", "Bold ", "Italic "};
    const int kw_flags[] = {GAI_TEXT_NORMAL, GAI_TEXT_BOLD, GAI_TEXT_ITALIC};

    /* some settings */
    setup.temps_at_once = gai_load_int_with_default("common/temps_at_once", 3);
    setup.orientation   = gai_load_int_with_default("common/orient", HORIZONTAL);

    setup.temp_unit     = gai_load_int_with_default("common/temp_unit", 
						    TEMP_UNIT_CELSIUS);

    setup.border = gai_load_int_with_default ("common/border", 3);
    setup.spacing = gai_load_int_with_default ("common/spacing", 3);


    /* load colors */

    /* color for normal temperature */
    guint32 col = gai_load_int_with_default ("common/col_okay", d_col_okay);

    setup.col_okay.r = GET_COL_R(col);
    setup.col_okay.g = GET_COL_G(col);
    setup.col_okay.b = GET_COL_B(col);
    setup.col_okay.alpha = GET_COL_A(col);

    /* color for high temperature */
    col = gai_load_int_with_default ("common/col_high", d_col_high);

    setup.col_high.r = GET_COL_R(col);
    setup.col_high.g = GET_COL_G(col);
    setup.col_high.b = GET_COL_B(col);
    setup.col_high.alpha = GET_COL_A(col);

    /* color for critical temperature */
    col = gai_load_int_with_default ("common/col_crit", d_col_crit);

    setup.col_critical.r = GET_COL_R(col);
    setup.col_critical.g = GET_COL_G(col);
    setup.col_critical.b = GET_COL_B(col);
    setup.col_critical.alpha = GET_COL_A(col);

    /* background color */
    col = gai_load_int_with_default ("common/col_bg", d_col_bg);

    setup.col_bg.r = GET_COL_R(col);
    setup.col_bg.g = GET_COL_G(col);
    setup.col_bg.b = GET_COL_B(col);
    setup.col_bg.alpha = 255; /* can't be transparent */

    /* use the gtk background? */
    setup.col_bg_use_gtk = gai_load_bool_with_default ("common/gtk_bg", TRUE);
    


    for (i=0; i < TEMP_SOURCES; i++)
    {
	/* load temperature limits */
	temp = g_strdup_printf ("%s/high_temp",  TEMP_SOURCE_NAMES[i]);
	setup.temp_delim[i][0] =  
	    gai_load_float_with_default (temp, TEMP_DELIM_HIGH_DEFAULT[i]);
	g_free (temp);

	temp = g_strdup_printf ("%s/critical_temp",  TEMP_SOURCE_NAMES[i]);
	setup.temp_delim[i][1] =  	    
	    gai_load_float_with_default (temp,  TEMP_DELIM_CRITICAL_DEFAULT[i] );
	g_free (temp);

	/* load switches */
	temp = g_strdup_printf ("%s/enable",  TEMP_SOURCE_NAMES[i]);
	setup.enable_temp_source[i] =
	    gai_load_bool_with_default (temp, TRUE);
	g_free(temp);

	/* set the default icon for the device */
	setup.icon [i] = img_default[i];
    } /* for all sources */


    /* load font settings */
    
    g_free (font_selected);

    font_selected = gai_load_string_with_default ("common/font", "");

    if (strcmp (font_selected, "") == 0)
      {
	/* if no font was selected */
	setup.use_default_font = 1;
      }
    else
      {
	setup.use_default_font = gai_load_bool_with_default ("common/default_font", 
							     TRUE);

    	/* reset pointers */
	flag_end = font_selected;
	flag_start = font_selected + strlen(font_selected);


	/* look for each keyword */
	
	for (n=0; n < KEYWORDS; n++)
	  {
	    temp = g_strrstr (font_selected, keywords[n]);
	    
	    if ( temp > font_selected )
	      {
		/* we found this flag */
		flags |= kw_flags[n];
		
		k = strlen (keywords[n]);
		
		flag_start = (temp < flag_start) ? temp -1 : flag_start;
		flag_end   = (temp + k +1> flag_end) ? temp + k : flag_end;
	      }
	  } /* for */
	
	if (flag_end != font_selected)
	    setup.font_size = atoi(flag_end);
	else
	    setup.font_size = atoi(g_strrstr (font_selected, " "));

	setup.font_feature = flags;

	g_free (setup.font_name);
	setup.font_name = g_strndup (font_selected, flag_start - font_selected);

      }  /* if font selected */

    /* allocate memory for the display struct */
    setup.temp_img = (temp_image_t*) malloc ( sizeof(temp_image_t)
					      *setup.temps_at_once );

    if (!setup.temp_img) {
		fprintf (stdlog, _("FATAL: unable to alloc memory"));
		exit (0);
    }

    /* put the selected sensors into a g_slist */

    g_slist_free (setup.sl_display); /* does this also free the data ? */
    setup.sl_display = NULL;
    
    /*
      - sensortype/dev_names (glist of sensor names of this group)
      
      - sensor_dev_name/display (boolean)
      - sensor_dev_name/icon		 (string)
      - sensor_dev_name/alias	 (string)
    */  
    
    
    for (i=0; i < TEMP_SOURCES; i++)
    {
	/* load list of sensors that belong to the
	   current group TEMP_SOURCE_NAMES[i] */
   	config_key = g_strdup_printf ("%s/%s", TEMP_SOURCE_NAMES[i], CONFIG_DEV_NAMES);
	p = sensor_list = gai_load_glist_with_default (config_key, NULL);
	g_free (config_key);
	
	
	while (p)
	{ 
	    if (p->data)
	    {
		char *device =  (char*) p->data;
		
		if ( (q = g_slist_find_custom (setup.sl_source[i], p->data, 
					       (GCompareFunc)&comp_sSensorTemp_str  )) )
		{
		    gchar **vect = NULL;
		    /* found a sensor with this device name */
		    sSensorTemp *stemp = S_SENSOR_TEMP(q->data);
		    
		    /* load display_value */
		    config_key = g_strdup_printf ("%s/%s", device, CONFIG_SENSOR_DISPLAY);
		    stemp->display = gai_load_bool_with_default (config_key, FALSE);
		    g_free (config_key);	   		
		    
		    if (stemp->display)
		    {
			/* add to display list */
			setup.sl_display = g_slist_append (setup.sl_display, stemp);
			/* switch on temp source, as a temp of this
			   group shall be displayed 
			*/
			setup.enable_temp_source[i] = TRUE; 
		    }
				
		    /* load alias name */
		    config_key = g_strdup_printf ("%s/%s", device, CONFIG_SENSOR_ALIAS);
		    vect = g_strsplit ( device, DEV_DELIM, 2);
		    /* try to get the name from the device-name */
		    if (!vect)
			temp = gai_load_string_with_default (config_key, 
							     device);
		    else
		    {
			temp=gai_load_string_with_default (config_key,vect[1]);
			g_strfreev (vect);
		    }
		    g_free (config_key);	   		
		    
		    if (temp)
		    {
			g_free (stemp->alias);	/* free old alias (if exist) */
			stemp->alias = temp; /* is the memory for temp allocated? */
		    } 
		    
		    
		    /* save icon name */
		    config_key = g_strdup_printf ("%s/%s", device, CONFIG_SENSOR_ICON);
		    temp = gai_load_string_with_default(config_key, "default");
		    
		    g_free (config_key);	   		
		    
		    if (strcmp (temp, "default") != 0)
		    {
			g_free (stemp->data);	/* free old icon name (if exist) */
			stemp->data = (void*)temp; /* is the memory for temp allocated? */
		    } 
		    else {
			g_free (temp);
		    }
		    
		    g_free (p->data); /* free device name */
		    p->data = NULL;
		} /* if sensor found by device name */
	    } /* if p->data */
	    
	    p = g_list_next(p);
	} /* while */
	
	if (sensor_list) g_list_free (sensor_list);
	
    } /* for */
    
    if (!setup.sl_display)
    {
	/* no temperatures selected to display ?*/
	for (i=0; i < TEMP_SOURCES; i++)
	{
	    GSList *cpy = g_slist_copy (setup.sl_source[i]);
	    
	    setup.sl_display = g_slist_concat (setup.sl_display, cpy);
	    
	} /* for */
	
	
	q = setup.sl_display;
	while (q)
	{
	    /* set the display flag of all sensors to TRUE */
	    S_SENSOR_TEMP(q->data)->display = TRUE;
	    q = g_slist_next(q);
	} /* while */
    } /* if */
    setup.sl_display_pos = setup.sl_display;
    
} /* load_config */

/************************************************************/



void gai_temp_settings(void)
{
    /* save the settings to disk */
    guint32 col, i;
    GSList *q;
    GList *save;
    gchar *config_key;

    int n, k, flags = 0;
    char *flag_start = NULL, *flag_end = NULL, *temp = NULL;

    const int KEYWORDS = 3;
    const char *keywords[] = {"Regular ", "Bold ", "Italic "};
    const int kw_flags[] = {GAI_TEXT_NORMAL, GAI_TEXT_BOLD, GAI_TEXT_ITALIC};


    /* some settings */
    gai_save_int ("common/temps_at_once", setup.temps_at_once);
    gai_save_int ("common/orient", setup.orientation);
    gai_save_int ("common/temp_unit", setup.temp_unit);

    gai_save_int ("common/border", setup.border);
    gai_save_int ("common/spacing", setup.spacing);

    /* save colors */

    /* color for normal temperature */
    col =  COL_STRUCT_INT32 (setup.col_okay);
    gai_save_int ("common/col_okay", col);

    /* color for high temperature */
    col =  COL_STRUCT_INT32 (setup.col_high);
    gai_save_int ("common/col_high", col);

    /* color for critical temperature */
    col =  COL_STRUCT_INT32 (setup.col_critical);
    gai_save_int ("common/col_critical", col);


    /* background color */
    col = COL_STRUCT_INT32 (setup.col_bg);
    gai_save_int ("common/col_bg", col);


    /* use the gtk background? */
    gai_save_bool ("common/gtk_bg", setup.col_bg_use_gtk);
    

    /* font settings */

    gai_save_bool ("common/default_font", setup.use_default_font);

    if (font_selected) 
      {
	/* save font */
	gai_save_string  ("common/font", font_selected);


	/* reset pointers */
	flag_end = font_selected;
	flag_start = font_selected + strlen(font_selected);

	/* look for each keyword */

	for (n=0; n < KEYWORDS; n++)
	  {
	    temp = g_strrstr (font_selected, keywords[n]);
	    
	    if ( temp > font_selected )
	      {
		/* we found this flag */
		flags |= kw_flags[n];
		fprintf (stdlog, "flag %s used ", keywords[n]);
		
		k = strlen (keywords[n]);
		
		flag_start = (temp < flag_start) ? temp -1 : flag_start;
		flag_end   = (temp + k +1> flag_end) ? temp + k : flag_end;
	      }
	  } /* for */

	if (flag_end && strlen (flag_end) > 0)
	  {
	    if (flag_end && (flag_end != font_selected) )
	      setup.font_size = atoi(flag_end);
	    else
	      setup.font_size = atoi(g_strrstr (font_selected, " "));
	  }
	else
	  {
	    setup.font_size = 10; /* default size */
	  }
	setup.font_feature = flags;

	g_free (setup.font_name);
	setup.font_name = g_strndup (font_selected, flag_start - font_selected);

      } /* font_selected */



    for (i=0; i < TEMP_SOURCES; i++)
    {
	/* save temperature limits */
	config_key = g_strdup_printf ("%s/high_temp",  TEMP_SOURCE_NAMES[i]);
	gai_save_float (config_key, setup.temp_delim[i][0]);
	g_free (config_key);

	config_key = g_strdup_printf ("%s/critical_temp",  TEMP_SOURCE_NAMES[i]);
	gai_save_float (config_key, setup.temp_delim[i][1]);;
	g_free (config_key);

	/* save switches */
	config_key = g_strdup_printf ("%s/enable",  TEMP_SOURCE_NAMES[i]);
	gai_save_bool (config_key, setup.enable_temp_source[i]);
	g_free(config_key);

    } /* for all sources */


    /* save a list of all sensors to each category they belong*/
    /*
      - sensortype/dev_names (glist of sensor names of this group)
      
      - sensor_dev_name/display (boolean)
      - sensor_dev_name/icon		 (string)
      - sensor_dev_name/alias	 (string)
    */  

    /* recreate display list */
    g_slist_free (setup.sl_display); /* does this also free the data ? */
    setup.sl_display = NULL;
    
    for (i=0; i < TEMP_SOURCES; i++)
    {
	save = NULL; /* reset the list of dev-names to save */
	q = setup.sl_source[i];

	while (q)
	{ 
	    /* for each temp source do */
	    if (q->data)
	    {
		char *device =  S_SENSOR_TEMP(q->data)->dev;
		
		
		save = g_list_append (save, device);

		/* save display_value */
		config_key = g_strdup_printf ("%s/%s", device, CONFIG_SENSOR_DISPLAY);
		gai_save_bool (config_key, S_SENSOR_TEMP(q->data)->display);
		g_free (config_key);	   		

		/* prepend to display list if selected*/
		if (S_SENSOR_TEMP(q->data)->display)
		    setup.sl_display = g_slist_append (setup.sl_display, 
						       q->data);
		
		/* save alias name */
		config_key = g_strdup_printf ("%s/%s", device, CONFIG_SENSOR_ALIAS);
		gai_save_string (config_key, S_SENSOR_TEMP(q->data)->alias);
		g_free (config_key);	   		
		
		/* save icon name */
		
		if ( S_SENSOR_TEMP(q->data)->data) 
		{
		    /* if we have an icon selected save it */
		    config_key = g_strdup_printf ("%s/%s", device, CONFIG_SENSOR_ICON);
		    gai_save_string (config_key, S_SENSOR_TEMP(q->data)->data);
		    g_free (config_key);	   		
		}
		
	    } /* if q->data */
	    
	    q = g_slist_next(q);
	} /* while */
	
	if (save)
	{
	    config_key = g_strdup_printf ("%s/%s", TEMP_SOURCE_NAMES[i], CONFIG_DEV_NAMES);
	    gai_save_glist (config_key, save);
	    g_free (config_key);
	    
	    g_list_free (save);
	} /* if save */
    } /* for */

    /* reset display positon */
    setup.sl_display_pos = setup.sl_display;


} /* gai_temp_settings */

/************************************************************/

/* the drawing function */
void draw_applet (GdkPixbuf *bg)
{
    int i, cur_x = 0, cur_y = 0, max_ico_txt = 0;
    temp_image_t *ti = NULL;


    draw_background (bg); /* clear the background */

    if (ENV_GNOME_PANEL == setup.env ) 
    {
	/* FIXME:
	   center the applet in its window */
	
	for (i=0; i < setup.temps_at_once; i++)
	{
	    ti = &setup.temp_img[i];

	    if (!ti->text)
	    {
		/* nothing more to draw */
		break;
	    }
	    
	    /* border offset */
	    cur_x += setup.border;
//	cur_y += setup.border; /* leave out the border */
	    
	    max_ico_txt =  MAX(ti->txt_y, ti->ico_y);

	    /* draw the icon */   
	    /*    gdk_pixbuf_copy_area (ti->icon, 0, 0, ti->ico_x, ti->ico_y, 
				  bg, cur_x, cur_y + (max_ico_txt-ti->ico_y)/2.0);
	    */
	    gdk_pixbuf_composite (ti->icon, bg, cur_x, 
				  cur_y + (max_ico_txt-ti->ico_y)/2.0,
				  ti->ico_x, ti->ico_y,
				  cur_x,
				  cur_y + (max_ico_txt-ti->ico_y)/2.0,
				  1.0, 1.0,
				  GDK_INTERP_NEAREST,
				  255);

	    
	    /* (max_ico_txt-ti->ico_y)/2.0 is the offset to center the 
	   icon vertically */
	    
	    /* draw the temp */
	    cur_x += setup.spacing + ti->ico_x; /* remind spacing */
/*	    
	    gdk_pixbuf_copy_area (ti->text, 0, 0, ti->txt_x, ti->txt_y,
				  bg, cur_x, cur_y + (max_ico_txt-ti->txt_y)/2.0 );
*/
	    gdk_pixbuf_composite (ti->text, bg, cur_x, 
				  cur_y + (max_ico_txt-ti->txt_y)/2.0,
				  ti->txt_x, ti->txt_y,
				  cur_x,
				  cur_y + (max_ico_txt-ti->txt_y)/2.0,
				  1.0, 1.0,
				  GDK_INTERP_NEAREST,
				  255);
	    
	    /* reset drawing position */
	    
	    if (setup.orientation == HORIZONTAL)
	    {
		cur_y = 0;
		cur_x += ti->txt_x + setup.border;
	    }
	    else 
	    {
		cur_x = 0;
		cur_y += setup.border + max_ico_txt;
	    } /* else (VERTICAL) */
	    
	} /* for */
    } /* ENV_GNOME_PANEL */
    else
    {
	/* the only difference by now is the border around each 
	   entry --> gai-temp.h */

	/* FIXME:
	   center the applet in its window */
	
	for (i=0; i < setup.temps_at_once; i++)
	{
	    ti = &setup.temp_img[i];

	    if (!ti->text)
	    {
		/* nothing more to draw */
		break;
	    }
	    
	    /* border offset */
	    cur_x += setup.border;
	    cur_y += setup.border;
	    
	    max_ico_txt =  MAX(ti->txt_y, ti->ico_y);
	    /* draw the icon */
	    gdk_pixbuf_composite (ti->icon, bg, cur_x, 
				  cur_y + (max_ico_txt-ti->ico_y)/2.0,
				  ti->ico_x, ti->ico_y,
				  cur_x,
				  cur_y + (max_ico_txt-ti->ico_y)/2.0,
				  1.0, 1.0,
				  GDK_INTERP_NEAREST,
				  255);
	   
	    
	    /* (max_ico_txt-ti->ico_y)/2.0 is the offset to center the 
	   icon vertically */
	    
	    /* draw the temp */
	    cur_x += setup.spacing + ti->ico_x; /* remind spacing */
	    
	   
	    gdk_pixbuf_composite (ti->text, bg, cur_x, 
				  cur_y + (max_ico_txt-ti->txt_y)/2.0,
				  ti->txt_x, ti->txt_y,
				  cur_x,
				  cur_y + (max_ico_txt-ti->txt_y)/2.0,
				  1.0, 1.0,
				  GDK_INTERP_NEAREST,
				  255);
 
	    /* reset drawing position */
	    
	    if (setup.orientation == HORIZONTAL)
	    {
		cur_y = 0;
		cur_x += ti->txt_x + setup.border;
	    }
	    else 
	    {
		cur_x = 0;
		cur_y += setup.border + max_ico_txt;
	    } /* else (VERTICAL) */
	    
	} /* for */
    } /* else not GNOME-VERSION */
    
    if (i == 0)  /* for-loop aborted */
    {
      int width, height;
	/* didn't draw anything, draw dummy image */
			
	if (!img_default[NO_SENSORS])
	    img_default[NO_SENSORS] = 
		gai_load_image_at_size ("null.svg", 
					setup.bg_dim.x, 
					setup.bg_dim.y);

		   
	width =  gdk_pixbuf_get_width  ( img_default[NO_SENSORS]);
	height = gdk_pixbuf_get_height ( img_default[NO_SENSORS]);

	gdk_pixbuf_composite (img_default[NO_SENSORS], 
			      bg, (setup.bg_dim.x -  width)/2, 
			      (setup.bg_dim.y - height)/2, 
			      width, height, 0, 0,
			      1.0, 1.0,
			      GDK_INTERP_NEAREST,
			      255);
    }
} /* draw applet */

/************************************************************/

void calc_applet_size (int *x, int *y)
{
    int i, sum_x = 0, sum_y = 0, max_x = 0, max_y = 0, tx, ty;
    temp_image_t *ti;

    if (!x || !y) 
    {
	fprintf (stdlog , _("ERROR: NULL as param in calc_applet_size\n"));
	return;
    } /* if wrong param */

    if (ENV_GNOME_PANEL == setup.env ) 
    {

	for (i=0; i < setup.temps_at_once; i++)
	{
	    ti = &setup.temp_img[i];
	    
	    if (!ti->text)
		break; /* no more images */
	    
	    tx = 2 * setup.border + setup.spacing + ti->ico_x + ti->txt_x;
	    ty = MAX (ti->ico_y, ti->txt_y);
	    
	    sum_y += ty;
	    sum_x += tx;
	    
	    max_x = MAX(max_x, tx);
	    max_y = MAX(max_y, ty);
	} /* for */
	
	if (setup.orientation == HORIZONTAL)
	{
	    *x = sum_x ; *y = max_y;
	} 
	else
	{
	    *x = max_x; *y = sum_y;
	}
    } /* ENV_GNOME_PANEL */
    else
    {
	/* the only difference by now: the border around 
	   each entry --> gai-temp.h */

	for (i=0; i < setup.temps_at_once; i++)
	{
	    ti = &setup.temp_img[i];

	    if (!ti->text)
		break; /* no more images */

	    
	    tx = 2 * setup.border + setup.spacing + ti->ico_x + ti->txt_x;
	    ty = 2 * setup.border + MAX (ti->ico_y, ti->txt_y);
	    
	    sum_y += ty;
	    sum_x += tx;
	    
	    max_x = MAX(max_x, tx);
	    max_y = MAX(max_y, ty);
	} /* for */
	
	if (setup.orientation == HORIZONTAL)
	{
	    *x = sum_x ; *y = max_y;
	} 
	else
	{
	    *x = max_x; *y = sum_y;
	}
    } /* NOT_GNOME_PANEL */

    if (*x == 0 || *y == 0)
    {	
	/* we probably don't have a sensor to read from 
	   display the dummy image */
	*x = 24;
	*y = 24;
    }
} /* calc_applet_size */

/************************************************************/
/* and now some small helper functions                      */
/************************************************************/

void draw_background (GdkPixbuf *bg)
{
    GtkWidget *drawingarea = gai_get_drawingarea();
    gint w, h;

    if (!bg) return;

    w = gdk_pixbuf_get_width  (bg);
    h = gdk_pixbuf_get_height (bg);

    
    if ( (ENV_GNOME_PANEL == setup.env) || setup.col_bg_use_gtk )
    {
	
	if (drawingarea && drawingarea->window)
	{
	    /* if the drawingarea is already created */
	    GdkPixmap *pixmap = gdk_pixmap_new  (drawingarea->window,
						 w, h,-1);
	    
	    GdkRectangle area = {0, 0, w, h};
	    
	    gtk_style_apply_default_background (drawingarea->style, 
						pixmap, TRUE,
						GTK_STATE_NORMAL, &area, 0, 0, 
						w, h);
	    
	    gdk_pixbuf_get_from_drawable  (bg, pixmap, NULL, 
					   0, 0, 0, 0, w, h);
	    
	    g_object_unref (pixmap);
	} /* if drawingarea created */
    }
    else
    {
	/* none-gnome-drawing version */
	gdk_pixbuf_fill (bg, COL_STRUCT_INT32(setup.col_bg));
    }
} /* draw_background */
/************************************************************/

int get_sensor_type_from_stemp (sSensorTemp *stemp)
{
    int i;

    /* detect to what type of sensor stemp belongs */

    for (i=0; i < TEMP_SOURCES; i++)
    {
	if ( g_slist_find(setup.sl_source[i], stemp) )
	{
	    /* found the data in that list */
	    return i;
	} /* if found */
    } /* for */

    return 0; /* not found - return default */

} /* get_sensor_type_from_stemp */

/************************************************************/

GdkPixbuf * get_icon_from_stemp (sSensorTemp *stemp)
{
    /* TODO:
       expand this function to load user-defined icons.

       They should be cached somewhere
    */

    int i = get_sensor_type_from_stemp (stemp);

    return setup.icon[i];
} /* get_icon_from_stemp */

/************************************************************/

GaiColor get_color_from_temp (sSensorTemp *stemp)
{
    /* this tries to find out what color to use */
    int i = get_sensor_type_from_stemp (stemp);

    float *delim_pair = setup.temp_delim[i]; 



    if (stemp->temp < delim_pair[I_HIGH_TEMP])
    {
	/* normal temperature */
	return setup.col_okay;
    }
    else
    {
	if (stemp->temp < delim_pair[I_CRITICAL_TEMP])
	    return setup.col_high;
	else
	    return setup.col_critical;
    }
} /* get_color_from_temp */

/************************************************************/

char *create_temp_str(sSensorTemp *stemp)
{
    char *ret;
    char *unit;
    float temp;
    
    switch (setup.temp_unit)
    { 
    case TEMP_UNIT_KELVIN:
	unit = "K";
	temp = stemp->temp + 273.2; /* better than 273.15 in this case ;-) */
	break;
    case TEMP_UNIT_FAHRENHEIT:
	unit = "°F"; 
	temp = stemp->temp * 1.8 + 32;
	break;
    default:
	unit = "°C";
	temp = stemp->temp; /* it is already in celsius */
	break;
    } /* switch */

    ret = g_strdup_printf ("%0.1f %s ", temp, unit);
    
    return ret;
    
} /* create_temp_str */

/************************************************************/

void select_font ()
{
    /* change the font of the applet text */
    GtkWidget*  font_dialog;
    int result;
    char *str;    

    font_dialog = gtk_font_selection_dialog_new (_("select a font please"));

    if (font_selected)
	gtk_font_selection_dialog_set_font_name
	    (GTK_FONT_SELECTION_DIALOG(font_dialog),
	     font_selected);

    
    result = gtk_dialog_run (GTK_DIALOG(font_dialog));

    str = gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG(font_dialog));

    g_free (font_selected); /* free it, if it contains something */

    font_selected = g_strdup (str);

    

    gtk_widget_destroy (font_dialog);


} /* select_font */

/************************************************************/
