
#ifndef READTEMP_H
#define READTEMP_H

#include <glib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#define ACPI_CPU_PATH "/proc/acpi/thermal_zone/THRM/temperature:" \
                      "/proc/acpi/thermal_zone/THM/temperature:" \
                      "/proc/acpi/thermal_zone/TZ1/temperature:" \
		      "/proc/acpi/thermal_zone/THM0/temperature"


#define DEV_PREFIX_ACPI_CPU "acpi_cpu"
#define DEV_PREFIX_HDDTEMP  "hddtemp"
#define DEV_PREFIX_LM_SENSORS "lm_sensor"

#define DEV_DELIM				 "::"


#define LINUX_2_6_SYSFS_I2C_PATH "/sys/devices/platform"
#define LINUX_2_6_SYSFS_I2C_PATH_LEN strlen(LINUX_2_6_SYSFS_I2C_PATH)

#define LINUX_2_6_I2C_PREFIX "i2c-"

#define LINUX_2_6_I2C_TEMP_FIRST "temp1_input"
#define LINUX_2_6_I2C_TEMP_MASK "temp%i_input"

#define HDD_TEMP_HOST_DEFAULT "localhost"
#define HDD_TEMP_PORT_DEFAULT 7634

/***************************************************
 * What about multiple processors or the P4 with   *
 * Hyper Threading concerning acpi ?               *
 *                                                 *
 * If anybody has such a thing, could you please   *
 * send me the content of                          *
 * /proc/acpi/thermal_zone/THRM/temperature        *
 * and the output of find /proc/acpi/thermal_zone  *
 * Thanks a lot !!                                 *
 ***************************************************/


/************************************************************
 * Data types                                               *
 ************************************************************/

enum 
{
    WELL_DONE = 0,
    ERROR_PARAM,
    ERROR_MEMORY,
    ERROR_READING,
    ERROR_PARSE
} MY_STATE;


typedef struct _sSensorTemp_
{
    char *dev;   /* a (unique) name of the device */
    char *alias; /* some nice name to display*/
    char *additional; /* more info from the source */

    float temp;  /* temperature in celsius (kelvin + 273.15 K) */

	 int  display; /* show this temperature */

    void *data;  /* some data to store */
} sSensorTemp;

#define S_SENSOR_TEMP(sensor) ((sSensorTemp*)sensor)

typedef GSList* (*READ_TEMP_FUNC_T) (GSList*, int*);

/************************************************************
 * Functions                                                *
 ************************************************************/

sSensorTemp *new_sSensorTemp(const char *dev, const char *alias, 
			     const char *add, float temp);

void free_sSensorTemp(sSensorTemp *sensor);

void dump_sSensorTemp (sSensorTemp* sensor, void *reserved);
/*
  I use "void *reserved" here to be compatible with GFunc,
  which is needed for g_slist_foreach
*/

int comp_sSensorTemp_str (void* sensor, void* str);
/* compare the name of a sSensorTemp element with a string */


char * query_hddtemp_deamon(const char * server, unsigned int port);
/* from gkrellm-hddtemp plugin */


/* Temperature reading */
GSList *read_cpu_temp_acpi (GSList *s_list, int *state);
GSList *read_hdd_temp (GSList *s_list, int *state);
GSList *read_lm_sensor_temp (GSList *s_list, int *state);

#endif /* READTEMP_H */
